@@ -1,5 +1,4 @@
 /* Dialog box implementation. */
-/* $Id: dialog.c,v 1.188.4.4 2005/04/06 11:00:57 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -15,7 +14,7 @@
 #include "config/options.h"
 #include "intl/gettext/libintl.h"
 #include "terminal/draw.h"
-#include "lowlevel/select.h"
+#include "main/timer.h"
 #include "terminal/kbd.h"
 #include "terminal/terminal.h"
 #include "terminal/window.h"
@@ -27,7 +26,7 @@
 #include "util/string.h"
 
 
-static window_handler dialog_func;
+static window_handler_T dialog_func;
 
 struct dialog_data *
 do_dialog(struct terminal *term, struct dialog *dlg,
@@ -58,7 +57,9 @@ update_all_widgets(struct dialog_data *dlg_data)
 {
 	struct widget_data *widget_data;
 
-	foreach_widget(dlg_data, widget_data) {
+	/* Iterate backwards rather than forwards so that listboxes are drawn
+	 * last, which means that they can grab the cursor. Yes, 'tis hacky. */
+	foreach_widget_back(dlg_data, widget_data) {
 		display_widget(dlg_data, widget_data);
 	}
 }
@@ -177,8 +178,12 @@ select_widget(struct dialog_data *dlg_data, struct widget_data *widget_data)
 struct widget_data *
 select_widget_by_id(struct dialog_data *dlg_data, int i)
 {
-	struct widget_data *widget_data = &dlg_data->widgets_data[i];
+	struct widget_data *widget_data;
 
+	if (i >= dlg_data->number_of_widgets)
+		return NULL;
+
+	widget_data = &dlg_data->widgets_data[i];
 	select_widget(dlg_data, widget_data);
 
 	return widget_data;
@@ -297,15 +302,15 @@ dialog_ev_kbd(struct dialog_data *dlg_data)
 	struct widget_data *widget_data = selected_widget(dlg_data);
 	struct widget_ops *ops = widget_data->widget->ops;
 	/* XXX: KEYMAP_EDIT ? --pasky */
-	enum menu_action action;
+	enum menu_action action_id;
 	struct term_event *ev = dlg_data->term_event;
 
 	/* First let the widget try out. */
 	if (ops->kbd && ops->kbd(dlg_data, widget_data) == EVENT_PROCESSED)
 		return;
 
-	action = kbd_action(KEYMAP_MENU, ev, NULL);
-	switch (action) {
+	action_id = kbd_action(KEYMAP_MENU, ev, NULL);
+	switch (action_id) {
 	case ACT_MENU_SELECT:
 		/* Can we select? */
 		if (ops->select) {
@@ -320,8 +325,8 @@ dialog_ev_kbd(struct dialog_data *dlg_data)
 		}
 
 		if (widget_is_textfield(widget_data)
-		    || check_kbd_modifier(ev, KBD_CTRL)
-		    || check_kbd_modifier(ev, KBD_ALT)) {
+		    || check_kbd_modifier(ev, KBD_MOD_CTRL)
+		    || check_kbd_modifier(ev, KBD_MOD_ALT)) {
 			select_button_by_flag(dlg_data, B_ENTER);
 		}
 		break;
@@ -358,8 +363,7 @@ dialog_ev_abort(struct dialog_data *dlg_data)
 	if (dlg_data->dlg->refresh) {
 		struct dialog_refresh *refresh = dlg_data->dlg->refresh;
 
-		if (refresh->timer != -1)
-			kill_timer(refresh->timer);
+		kill_timer(&refresh->timer);
 		mem_free(refresh);
 	}
 
@@ -436,7 +440,7 @@ check_dialog(struct dialog_data *dlg_data)
 	return 0;
 }
 
-t_handler_event_status
+widget_handler_status_T
 cancel_dialog(struct dialog_data *dlg_data, struct widget_data *xxx)
 {
 	delete_window(dlg_data->win);
@@ -459,10 +463,10 @@ update_dialog_data(struct dialog_data *dlg_data)
 	return 0;
 }
 
-t_handler_event_status
+widget_handler_status_T
 ok_dialog(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
-	t_done_handler *done = widget_data->widget->info.button.done;
+	done_handler_T *done = widget_data->widget->info.button.done;
 	void *done_data = widget_data->widget->info.button.done_data;
 
 	if (check_dialog(dlg_data)) return EVENT_NOT_PROCESSED;
@@ -474,7 +478,7 @@ ok_dialog(struct dialog_data *dlg_data, struct widget_data *widget_data)
 }
 
 /* Clear dialog fields (if widget has clear callback). */
-t_handler_event_status
+widget_handler_status_T
 clear_dialog(struct dialog_data *dlg_data, struct widget_data *xxx)
 {
 	struct widget_data *widget_data;
@@ -486,7 +490,7 @@ clear_dialog(struct dialog_data *dlg_data, struct widget_data *xxx)
 
 	/* Move focus to the first widget. It helps with bookmark search dialog
 	 * (and others). */
- 	select_widget_by_id(dlg_data, 0);
+	select_widget_by_id(dlg_data, 0);
 
 	redraw_dialog(dlg_data, 0);
 	return EVENT_PROCESSED;
@@ -618,7 +622,7 @@ do_refresh_dialog(struct dialog_data *dlg_data)
 
 	if (refresh_code == REFRESH_CANCEL
 	    || refresh_code == REFRESH_STOP) {
-		refresh->timer = -1;
+		refresh->timer = TIMER_ID_UNDEF;
 		if (refresh_code == REFRESH_CANCEL)
 			cancel_dialog(dlg_data, NULL);
 		return;
@@ -630,12 +634,12 @@ do_refresh_dialog(struct dialog_data *dlg_data)
 		redraw_dialog(dlg_data, 1);
 	}
 
-	refresh->timer = install_timer(RESOURCE_INFO_REFRESH,
-				(void (*)(void *)) do_refresh_dialog, dlg_data);
+	install_timer(&refresh->timer, RESOURCE_INFO_REFRESH,
+		      (void (*)(void *)) do_refresh_dialog, dlg_data);
 }
 
 void
-refresh_dialog(struct dialog_data *dlg_data, dialog_refresh_handler handler, void *data)
+refresh_dialog(struct dialog_data *dlg_data, dialog_refresh_handler_T handler, void *data)
 {
 	struct dialog_refresh *refresh = dlg_data->dlg->refresh;
 
@@ -645,12 +649,12 @@ refresh_dialog(struct dialog_data *dlg_data, dialog_refresh_handler handler, voi
 
 		dlg_data->dlg->refresh = refresh;
 
-	} else if (refresh->timer != -1) {
-		kill_timer(refresh->timer);
+	} else {
+		kill_timer(&refresh->timer);
 	}
 
 	refresh->handler = handler;
 	refresh->data = data;
-	refresh->timer = install_timer(RESOURCE_INFO_REFRESH,
-				(void (*)(void *)) do_refresh_dialog, dlg_data);
+	install_timer(&refresh->timer, RESOURCE_INFO_REFRESH,
+		      (void (*)(void *)) do_refresh_dialog, dlg_data);
 }
