@@ -1,5 +1,4 @@
 /* Listbox widget implementation. */
-/* $Id: listbox.c,v 1.183.4.5 2005/04/06 08:32:41 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,12 +26,12 @@
 void
 add_dlg_listbox(struct dialog *dlg, int height, void *box_data)
 {
-	struct widget *widget;
+	struct widget *widget = &dlg->widgets[dlg->number_of_widgets++];
 
-	widget = &dlg->widgets[dlg->number_of_widgets++];
 	widget->type = WIDGET_LISTBOX;
-	widget->info.listbox.height = height;
 	widget->data = box_data;
+
+	widget->info.listbox.height = height;
 }
 
 struct listbox_data *
@@ -48,33 +47,26 @@ dlg_format_listbox(struct terminal *term, struct widget_data *widget_data,
 	           int x, int *y, int w, int max_height, int *rw,
 	           enum format_align align)
 {
-	int min, optimal_h;
-
-	set_box(&widget_data->box, x, *y, w, 1);
-
-	if (rw) int_bounds(rw, widget_data->box.width, w);
+	int min, optimal_h, height;
 
 	/* Height bussiness follows: */
 
-	/* We ignore this one happily now. Rather be Dynamic ;p. */
-	/* (*y) += widget_data->widget->gid; */
-
 	/* This is only weird heuristic, it could scale well I hope. */
 	optimal_h = max_height * 7 / 10 - VERTICAL_LISTBOX_MARGIN;
 	min = get_opt_int("ui.dialogs.listbox_min_height");
 
 	if (max_height - VERTICAL_LISTBOX_MARGIN < min) {
 		/* Big trouble: can't satisfy even the minimum :-(. */
-		widget_data->box.height = max_height - VERTICAL_LISTBOX_MARGIN;
+		height = max_height - VERTICAL_LISTBOX_MARGIN;
 	} else if (optimal_h < min) {
-		widget_data->box.height = min;
+		height = min;
 	} else {
-		widget_data->box.height = optimal_h;
+		height = optimal_h;
 	}
 
-	/* DBG("::%d(%d)::%d::%d::", max_y, term?1:2, widget_data->h, *y); */
-
-	(*y) += widget_data->box.height;
+	set_box(&widget_data->box, x, *y, w, height);
+	(*y) += height;
+	if (rw) *rw = w;
 }
 
 
@@ -264,33 +256,15 @@ done_down:
 	return visible_item;
 }
 
-/* Takes care about listbox top moving. */
 static int
-listbox_sel_move_do(struct listbox_item *item, void *data_, int *offset)
+calc_dist(struct listbox_item *item, void *data_, int *offset)
 {
-	struct listbox_context *data = data_;
+	int *item_offset = data_;
 
-	if (item == data->box->top)
-		data->box->sel_offset = 0; /* assure resync */
-
-	if (data->dist > 0) {
-		if (data->box->sel_offset
-		    < data->widget_data->box.height - 1) {
-			data->box->sel_offset++;
-		} else {
-			data->box->top =
-				traverse_listbox_items_list(data->box->top,
-					data->box, 1, 1, NULL, NULL);
-		}
-	} else if (data->dist < 0) {
-		if (data->box->sel_offset > 0) {
-			data->box->sel_offset--;
-		} else {
-			data->box->top =
-				traverse_listbox_items_list(data->box->top,
-					data->box, -1, 1, NULL, NULL);
-		}
-	}
+	if (*offset < 0)
+		--*item_offset;
+	else if (*offset > 0)
+		++*item_offset;
 
 	return 0;
 }
@@ -302,10 +276,10 @@ listbox_sel_move(struct widget_data *widget_data, int dist)
 {
 	struct listbox_data *box = get_listbox_widget_data(widget_data);
 
-	if (!list_empty(*box->items)) {
-		if (!box->top) box->top = box->items->next;
-		if (!box->sel) box->sel = box->top;
-	}
+	if (list_empty(*box->items)) return;
+
+	if (!box->top) box->top = box->items->next;
+	if (!box->sel) box->sel = box->top;
 
 	/* We want to have these visible if possible. */
 	if (box->top && !box->top->visible) {
@@ -314,22 +288,64 @@ listbox_sel_move(struct widget_data *widget_data, int dist)
 		box->sel = box->top;
 	}
 
-	if (traverse_listbox_items_list(box->sel, box, dist, 1, NULL, NULL)
-	    != box->sel) {
-		struct listbox_context data;
+	if (!dist && !box->sel->visible) dist = 1;
 
-		memset(&data, 0, sizeof(data));
-
-		data.box = box;
-		data.widget_data = widget_data;
-		data.dist = dist;
-
-		/* XXX: This is ugly, yes; but we don't want to call the
-		 * callback if we won't move on at all. */
+	if (dist) {
 		box->sel = traverse_listbox_items_list(box->sel, box, dist, 1,
-						       listbox_sel_move_do,
-						       &data);
+		                                       calc_dist,
+		                                       &box->sel_offset);
+		/* box->sel_offset becomes the offset of the new box->sel
+		 * from box->top. */
 	}
+
+	if (box->sel_offset < 0) {
+		/* We must scroll up. */
+		box->sel_offset = 0;
+		box->top = box->sel;
+	} else if (box->sel_offset >= widget_data->box.height) {
+		/* We must scroll down. */
+		box->sel_offset = widget_data->box.height - 1;
+		box->top = traverse_listbox_items_list(box->sel, box,
+					   1 - widget_data->box.height,
+					   1, NULL, NULL);
+	}
+}
+
+static int
+test_search(struct listbox_item *item, void *data_, int *offset)
+{
+	struct listbox_context *listbox_context = data_;
+
+	if (item != listbox_context->item)
+		listbox_context->offset++;
+	else
+		*offset = 0;
+
+	return 0;
+}
+
+static int
+listbox_item_offset(struct listbox_data *box, struct listbox_item *item)
+{
+	struct listbox_context ctx;
+
+	memset(&ctx, 0, sizeof(ctx));
+	ctx.item = item;
+	ctx.offset = 0;
+
+	traverse_listbox_items_list(box->items->next, box, 0, 1, test_search, &ctx);
+
+	return ctx.offset;
+}
+
+void
+listbox_sel(struct widget_data *widget_data, struct listbox_item *item)
+{
+	struct listbox_data *box = get_listbox_widget_data(widget_data);
+
+	listbox_sel_move(widget_data,
+	                 listbox_item_offset(box, item)
+	                  - listbox_item_offset(box, box->sel));
 }
 
 
@@ -338,18 +354,22 @@ static int
 display_listbox_item(struct listbox_item *item, void *data_, int *offset)
 {
 	struct listbox_context *data = data_;
-	unsigned char *stylename;
 	int len; /* Length of the current text field. */
-	struct color_pair *color;
+	struct color_pair *tree_color, *text_color;
 	int depth = item->depth + 1;
 	int d;
 	int x, y;
 
-	stylename = (item == data->box->sel) ? "menu.selected"
-		  : ((item->marked)	     ? "menu.marked"
-					     : "menu.normal");
+	tree_color = get_bfu_color(data->term, "menu.normal");
+	if (item == data->box->sel) {
+		text_color = get_bfu_color(data->term, "menu.selected");
 
-	color = get_bfu_color(data->term, stylename);
+	} else if (item->marked) {
+		text_color = get_bfu_color(data->term, "menu.marked");
+
+	} else {
+		text_color = tree_color;
+	}
 
 	y = data->widget_data->box.y + data->offset;
 	for (d = 0; d < depth - 1; d++) {
@@ -364,13 +384,13 @@ display_listbox_item(struct listbox_item *item, void *data_, int *offset)
 
 		/* XXX */
 		x = data->widget_data->box.x + d * 5;
-		draw_text(data->term, x, y, "     ", 5, 0, color);
+		draw_text(data->term, x, y, "     ", 5, 0, tree_color);
 
 		if (root ? root->child.prev == child
 			 : data->box->items->prev == child)
 			continue; /* We were the last branch. */
 
-		draw_border_char(data->term, x + 1, y, BORDER_SVLINE, color);
+		draw_border_char(data->term, x + 1, y, BORDER_SVLINE, tree_color);
 	}
 
 	if (depth) {
@@ -413,7 +433,7 @@ display_listbox_item(struct listbox_item *item, void *data_, int *offset)
 
 		x = data->widget_data->box.x + (depth - 1) * 5;
 		for (i = 0; i < 5; i++) {
-			draw_border_char(data->term, x + i, y, str[i], color);
+			draw_border_char(data->term, x + i, y, str[i], tree_color);
 		}
 	}
 
@@ -424,7 +444,7 @@ display_listbox_item(struct listbox_item *item, void *data_, int *offset)
 		int width = data->widget_data->box.width - depth * 5;
 
 		for (i = 0; i < width; i++) {
-			draw_border_char(data->term, x + i, y, BORDER_SHLINE, color);
+			draw_border_char(data->term, x + i, y, BORDER_SHLINE, text_color);
 		}
 
 	} else if (data->box->ops && data->box->ops->draw) {
@@ -444,15 +464,14 @@ display_listbox_item(struct listbox_item *item, void *data_, int *offset)
 		len = strlen(text);
 		int_upper_bound(&len, int_max(0, data->widget_data->box.width - depth * 5));
 
-		draw_text(data->term, x, y, text, len, 0, color);
+		draw_text(data->term, x, y, text, len, 0, text_color);
 
 		mem_free(text);
 	}
 
 	if (item == data->box->sel) {
-		x = data->widget_data->box.x;
-
 		/* For blind users: */
+		x = data->widget_data->box.x + 5 + item->depth * 5;
 		set_cursor(data->term, x, y, 1);
 		set_window_ptr(data->dlg_data->win, x, y);
 	}
@@ -463,36 +482,18 @@ display_listbox_item(struct listbox_item *item, void *data_, int *offset)
 }
 
 /* Displays a dialog box */
-static t_handler_event_status
+static widget_handler_status_T
 display_listbox(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	struct terminal *term = dlg_data->win->term;
 	struct listbox_data *box = get_listbox_widget_data(widget_data);
 	struct listbox_context data;
-	/* Add one to offset to get the actual height of the selected item */
-	int moves = box->sel_offset + 1 - widget_data->box.height;
 
-	if (moves > 0) {
-		/* Move selected listbox to visible and update box->top while we're
-		 * at it. Fixes bug 58. */
-		listbox_sel_move(widget_data, -moves);
-
-	} else if (!list_empty(*box->items)) {
-		if (!box->top) box->top = box->items->next;
-		if (!box->sel) box->sel = box->top;
-	}
+	listbox_sel_move(widget_data, 0);
 
 	draw_box(term, &widget_data->box, ' ', 0,
 		 get_bfu_color(term, "menu.normal"));
 
-	/* We want to have these visible if possible. */
-	if (box->top && !box->top->visible) {
-		/* DBG("top: %s - (%d) %p\n", box->top->text, box->top->visible, box->top); */
-		box->top = traverse_listbox_items_list(box->top, box, 1,
-				1, NULL, NULL);
-		box->sel = box->top;
-	}
-
 	memset(&data, 0, sizeof(data));
 	data.term = term;
 	data.widget_data = widget_data;
@@ -521,7 +522,7 @@ check_old_state(struct listbox_item *item, void *info_, int *offset)
 	return 0;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 init_listbox(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	struct hierbox_browser *browser = dlg_data->dlg->udata2;
@@ -547,7 +548,7 @@ init_listbox(struct dialog_data *dlg_data, struct widget_data *widget_data)
 	return EVENT_PROCESSED;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 mouse_listbox(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 #ifdef CONFIG_MOUSE
@@ -606,93 +607,118 @@ mouse_listbox(struct dialog_data *dlg_data, struct widget_data *widget_data)
 	return EVENT_NOT_PROCESSED;
 }
 
-static t_handler_event_status
-kbd_listbox(struct dialog_data *dlg_data, struct widget_data *widget_data)
+static widget_handler_status_T
+do_kbd_listbox_action(enum menu_action action_id, struct dialog_data *dlg_data,
+		      struct widget_data *widget_data)
 {
 	struct widget_data *dlg_item = dlg_data->widgets_data;
+
+	switch (action_id) {
+		case ACT_MENU_DOWN:
+			listbox_sel_move(dlg_item, 1);
+			display_widget(dlg_data, dlg_item);
+
+			return EVENT_PROCESSED;
+
+		case ACT_MENU_UP:
+			listbox_sel_move(dlg_item, -1);
+			display_widget(dlg_data, dlg_item);
+
+			return EVENT_PROCESSED;
+
+		case ACT_MENU_PAGE_DOWN:
+		{
+			struct listbox_data *box;
+
+			box = get_listbox_widget_data(dlg_item);
+
+			listbox_sel_move(dlg_item,
+			                 2 * dlg_item->box.height
+			                   - box->sel_offset - 1);
+
+			display_widget(dlg_data, dlg_item);
+
+			return EVENT_PROCESSED;
+		}
+
+		case ACT_MENU_PAGE_UP:
+		{
+			struct listbox_data *box;
+
+			box = get_listbox_widget_data(dlg_item);
+
+			listbox_sel_move(dlg_item,
+			                 -dlg_item->box.height
+			                  - box->sel_offset);
+
+			display_widget(dlg_data, dlg_item);
+
+			return EVENT_PROCESSED;
+		}
+
+		case ACT_MENU_HOME:
+			listbox_sel_move(dlg_item, -INT_MAX);
+			display_widget(dlg_data, dlg_item);
+
+			return EVENT_PROCESSED;
+
+		case ACT_MENU_END:
+			listbox_sel_move(dlg_item, INT_MAX);
+			display_widget(dlg_data, dlg_item);
+
+			return EVENT_PROCESSED;
+
+		case ACT_MENU_MARK_ITEM:
+		{
+			struct listbox_data *box;
+
+			box = get_listbox_widget_data(dlg_item);
+			if (box->sel) {
+				box->sel->marked = !box->sel->marked;
+				listbox_sel_move(dlg_item, 1);
+			}
+			display_widget(dlg_data, dlg_item);
+
+			return EVENT_PROCESSED;
+		}
+
+		case ACT_MENU_DELETE:
+		{
+			struct listbox_data *box;
+
+			box = get_listbox_widget_data(dlg_item);
+			if (box->ops
+			    && box->ops->delete
+			    && box->ops->can_delete)
+				push_hierbox_delete_button(dlg_data,
+							   widget_data);
+
+			return EVENT_PROCESSED;
+		}
+
+		default:
+			break;
+	}
+
+	return EVENT_NOT_PROCESSED;
+}
+
+static widget_handler_status_T
+kbd_listbox(struct dialog_data *dlg_data, struct widget_data *widget_data)
+{
 	struct term_event *ev = dlg_data->term_event;
 
 	/* Not a pure listbox, but you're not supposed to use this outside of
 	 * the listbox browser anyway, so what.. */
 
 	switch (ev->ev) {
-		enum menu_action action;
-
 		case EVENT_KBD:
-			action = kbd_action(KEYMAP_MENU, ev, NULL);
-
-			/* Moving the box */
-			if (action == ACT_MENU_DOWN) {
-				listbox_sel_move(dlg_item, 1);
-				display_widget(dlg_data, dlg_item);
-
-				return EVENT_PROCESSED;
-			}
-
-			if (action == ACT_MENU_UP) {
-				listbox_sel_move(dlg_item, -1);
-				display_widget(dlg_data, dlg_item);
-
-				return EVENT_PROCESSED;
-			}
-
-			if (action == ACT_MENU_PAGE_DOWN) {
-				listbox_sel_move(dlg_item, dlg_item->box.height / 2);
-				display_widget(dlg_data, dlg_item);
-
-				return EVENT_PROCESSED;
-			}
-
-			if (action == ACT_MENU_PAGE_UP) {
-				listbox_sel_move(dlg_item, -dlg_item->box.height / 2);
-				display_widget(dlg_data, dlg_item);
-
-				return EVENT_PROCESSED;
-			}
-
-			if (action == ACT_MENU_HOME) {
-				listbox_sel_move(dlg_item, -INT_MAX);
-				display_widget(dlg_data, dlg_item);
-
-				return EVENT_PROCESSED;
-			}
-
-			if (action == ACT_MENU_END) {
-				listbox_sel_move(dlg_item, INT_MAX);
-				display_widget(dlg_data, dlg_item);
-
-				return EVENT_PROCESSED;
-			}
-
-			if (action == ACT_MENU_MARK_ITEM) {
-				struct listbox_data *box;
-
-				box = get_listbox_widget_data(dlg_item);
-				if (box->sel) {
-					box->sel->marked = !box->sel->marked;
-					listbox_sel_move(dlg_item, 1);
-				}
-				display_widget(dlg_data, dlg_item);
-
-				return EVENT_PROCESSED;
-			}
-
-			if (action == ACT_MENU_DELETE) {
-				struct listbox_data *box;
-
-				box = get_listbox_widget_data(dlg_item);
-				if (box->ops
-				    && box->ops->delete
-				    && box->ops->can_delete)
-					push_hierbox_delete_button(dlg_data,
-								   widget_data);
-
-				return EVENT_PROCESSED;
-			}
-
-			/* Selecting a button; most probably ;). */
-			break;
+		{
+			enum menu_action action_id;
 
+			action_id = kbd_action(KEYMAP_MENU, ev, NULL);
+			return do_kbd_listbox_action(action_id, dlg_data, widget_data);
+		}
 		case EVENT_INIT:
 		case EVENT_RESIZE:
 		case EVENT_REDRAW:
