@@ -1,5 +1,4 @@
 /* Text widget implementation. */
-/* $Id: text.c,v 1.116.4.1 2005/04/05 21:11:05 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,12 +25,12 @@ void
 add_dlg_text(struct dialog *dlg, unsigned char *text,
 	     enum format_align align, int bottom_pad)
 {
-	struct widget *widget;
+	struct widget *widget = &dlg->widgets[dlg->number_of_widgets++];
 
-	widget = &dlg->widgets[dlg->number_of_widgets++];
 	widget->type = WIDGET_TEXT;
 	widget->text = text;
-	widget->info.text.align = align;
+
+	widget->info.text.align	   = align;
 	widget->info.text.is_label = !!bottom_pad;
 }
 
@@ -182,17 +181,20 @@ dlg_format_text(struct terminal *term, struct widget_data *widget_data,
 	unsigned char *text = widget_data->widget->text;
 	unsigned char saved = 0;
 	unsigned char *saved_pos = NULL;
+	int height;
+
+	height = int_max(0, max_height - 3);
 
 	/* If we are drawing set up the box before setting up the
 	 * scrolling. */
 	set_box(&widget_data->box, x, *y,
-		widget_data->box.width, int_max(0, max_height - 3));
-	if (widget_data->box.height == 0) return;
+		widget_data->box.width, height);
+	if (height == 0) return;
 
 	/* Can we scroll and do we even have to? */
 	if (widget_data->widget->info.text.is_scrollable
 	    && (widget_data->info.text.max_width != width
-		|| widget_data->box.height < widget_data->info.text.lines))
+		|| height < widget_data->info.text.lines))
 	{
 		unsigned char **lines;
 		int current;
@@ -207,8 +209,8 @@ dlg_format_text(struct terminal *term, struct widget_data *widget_data,
 		lines = (unsigned char **) widget_data->cdata;
 
 		/* Make maximum number of lines available */
-		visible = int_max(widget_data->info.text.lines - widget_data->box.height,
-				  widget_data->box.height);
+		visible = int_max(widget_data->info.text.lines - height,
+				  height);
 
 		int_bounds(&widget_data->info.text.current, 0, visible);
 		current = widget_data->info.text.current;
@@ -218,8 +220,8 @@ dlg_format_text(struct terminal *term, struct widget_data *widget_data,
 
 		/* Do we have to force a text end ? */
 		visible = widget_data->info.text.lines - current;
-		if (visible > widget_data->box.height) {
-			int lines_pos = current + widget_data->box.height;
+		if (visible > height) {
+			int lines_pos = current + height;
 
 			saved_pos = lines[lines_pos];
 
@@ -252,7 +254,7 @@ dlg_format_text(struct terminal *term, struct widget_data *widget_data,
 	if (saved && saved_pos) *saved_pos = saved;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 display_text(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	struct window *win = dlg_data->win;
@@ -337,7 +339,7 @@ format_and_display_text(struct widget_data *widget_data,
 	redraw_from_window(dlg_data->win);
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 kbd_text(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 	int current = widget_data->info.text.current;
@@ -377,7 +379,7 @@ kbd_text(struct dialog_data *dlg_data, struct widget_data *widget_data)
 	return EVENT_PROCESSED;
 }
 
-static t_handler_event_status
+static widget_handler_status_T
 mouse_text(struct dialog_data *dlg_data, struct widget_data *widget_data)
 {
 #ifdef CONFIG_MOUSE
