@@ -82,9 +82,9 @@ get_cache_entry_info(struct listbox_item *item, struct terminal *term)
 		}
 	}
 
-	add_format_to_string(&msg, "\n%s: %" PRId64, _("Size", term),
+	add_format_to_string(&msg, "\n%s: %" OFF_T_FORMAT, _("Size", term),
 	                     cached->length);
-	add_format_to_string(&msg, "\n%s: %" PRId64, _("Loaded size", term),
+	add_format_to_string(&msg, "\n%s: %" OFF_T_FORMAT, _("Loaded size", term),
 						cached->data_size);
 	if (cached->content_type) {
 		add_format_to_string(&msg, "\n%s: %s", _("Content type", term),
