@@ -1,5 +1,3 @@
-/* $Id: conf.h,v 1.14 2004/02/04 23:13:56 pasky Exp $ */
-
 #ifndef EL__CONFIG_CONF_H
 #define EL__CONFIG_CONF_H
 
@@ -18,9 +16,15 @@ enum parse_error {
 void load_config(void);
 enum parse_error parse_config_command(struct option *options,
 				      unsigned char **file, int *line,
-				      struct string *mirror);
+				      struct string *mirror,
+				      int is_system_conf);
 void parse_config_file(struct option *options, unsigned char *name,
-		       unsigned char *file, struct string *mirror);
+		       unsigned char *file, struct string *mirror,
+		       int is_system_conf);
 int write_config(struct terminal *);
 
+unsigned char *
+create_config_string(unsigned char *prefix, unsigned char *name,
+		     struct option *options);
+
 #endif
