@@ -1,9 +1,8 @@
-/* $Id: parser.h,v 1.76.2.2 2005/05/01 22:47:55 jonas Exp $ */
 
 #ifndef EL__DOCUMENT_HTML_PARSER_H
 #define EL__DOCUMENT_HTML_PARSER_H
 
-
+#include "intl/charsets.h" /* unicode_val_T */
 #include "util/align.h"
 #include "util/color.h"
 #include "util/lists.h"
@@ -11,12 +10,11 @@
 struct document_options;
 struct form_control;
 struct frameset_desc;
+struct html_context;
 struct memory_list;
 struct menu_item;
 struct part;
-struct session;
 struct string;
-struct terminal;
 struct uri;
 
 /* XXX: This is just terible - this interface is from 75% only for other HTML
@@ -32,12 +30,14 @@ enum format_attr {
 	AT_SUBSCRIPT = 32,
 	AT_SUPERSCRIPT = 64,
 	AT_PREFORMATTED = 128,
+	AT_UPDATE_SUB = 256,
+	AT_UPDATE_SUP = 512,
 };
 
 struct text_attrib_style {
 	enum format_attr attr;
-	color_t fg;
-	color_t bg;
+	color_T fg;
+	color_T bg;
 };
 
 struct text_attrib {
@@ -49,17 +49,17 @@ struct text_attrib {
 	unsigned char *image;
 	unsigned char *title;
 	struct form_control *form;
-	color_t clink;
-	color_t vlink;
+	color_T clink;
+	color_T vlink;
 #ifdef CONFIG_BOOKMARKS
-	color_t bookmark_link;
+	color_T bookmark_link;
 #endif
-	color_t image_link;
+	color_T image_link;
 
 	unsigned char *select;
 	int select_disabled;
 	unsigned int tabindex;
-	long accesskey;
+	unicode_val_T accesskey;
 
 	unsigned char *onclick;
 	unsigned char *ondblclick;
@@ -98,16 +98,16 @@ struct par_attrib {
 	unsigned list_number;
 	int dd_margin;
 	enum format_list_flag flags;
-	color_t bgcolor;
+	color_T bgcolor;
 };
 
 /* HTML parser stack mortality info */
-enum html_element_type {
+enum html_element_mortality_type {
 	/* Elements of this type can not be removed from the stack. This type
-	 * is created by the renderer when formatting a HTML part. */
+	 * is created by the renderer when formatting an HTML part. */
 	ELEMENT_IMMORTAL,
 	/* Elements of this type can only be removed by elements of the start
-	 * type. This type is created whenever a HTML state is created using
+	 * type. This type is created whenever an HTML state is created using
 	 * init_html_parser_state(). */
 	/* The element has been created by*/
 	ELEMENT_DONT_KILL,
@@ -122,7 +122,7 @@ enum html_element_type {
 struct html_element {
 	LIST_HEAD(struct html_element);
 
-	enum html_element_type type;
+	enum html_element_mortality_type type;
 
 	struct text_attrib attr;
 	struct par_attrib parattr;
@@ -130,6 +130,8 @@ struct html_element {
 	unsigned char *name;
 	int namelen;
 	unsigned char *options;
+	/* See document/html/parser/parse.c's element_info.linebreak
+	 * description. */
 	int linebreak;
 	struct frameset_desc *frameset;
 
@@ -139,6 +141,8 @@ struct html_element {
 		ELEMENT_VISITED = 2,
 	} pseudo_class;
 };
+#define is_inline_element(e) (e->linebreak == 0)
+#define is_block_element(e) (e->linebreak > 0)
 
 enum html_special_type {
 	SP_TAG,
@@ -159,37 +163,48 @@ enum html_special_type {
 
 /* Interface for the renderer */
 
-void
+struct html_context *
 init_html_parser(struct uri *uri, struct document_options *options,
 		 unsigned char *start, unsigned char *end,
 		 struct string *head, struct string *title,
-		 void (*put_chars)(struct part *, unsigned char *, int),
-		 void (*line_break)(struct part *),
-		 void *(*special)(struct part *, enum html_special_type, ...));
+		 void (*put_chars)(struct html_context *, unsigned char *, int),
+		 void (*line_break)(struct html_context *),
+		 void *(*special)(struct html_context *, enum html_special_type,
+		                  ...));
 
-void done_html_parser(void);
-struct html_element *init_html_parser_state(enum html_element_type type, int align, int margin, int width);
-void done_html_parser_state(struct html_element *element);
+void done_html_parser(struct html_context *html_context);
+struct html_element *init_html_parser_state(struct html_context *html_context, enum html_element_mortality_type type, int align, int margin, int width);
+void done_html_parser_state(struct html_context *html_context,
+                            struct html_element *element);
 
 /* Interface for the table handling */
 
-int get_bgcolor(unsigned char *, color_t *);
-void set_fragment_identifier(unsigned char *attr_name, unsigned char *attr);
-void add_fragment_identifier(struct part *, unsigned char *attr);
+int get_bgcolor(struct html_context *html_context, unsigned char *a, color_T *rgb);
+void set_fragment_identifier(struct html_context *html_context,
+                             unsigned char *attr_name, unsigned char *attr);
+void add_fragment_identifier(struct html_context *html_context,
+                             struct part *, unsigned char *attr);
 
 /* Interface for the viewer */
 
 int
 get_image_map(unsigned char *head, unsigned char *pos, unsigned char *eof,
 	      struct menu_item **menu, struct memory_list **ml, struct uri *uri,
-	      unsigned char *target_base, int to, int def, int hdef);
+	      struct document_options *options, unsigned char *target_base,
+	      int to, int def, int hdef);
 
-/* For html/parser/forms.c,link.c,parse.c,stack.c */
+/* For html/parser/forms.c,general.c,link.c,parse.c,stack.c */
 
-void ln_break(int n, void (*line_break)(struct part *), struct part *part);
+/* Ensure that there are at least n successive line-breaks at the current
+ * position, but don't add more than necessary to bring the current number
+ * of successive line-breaks above n.
+ *
+ * For example, there should be two line-breaks after a <br>, but multiple
+ * successive <br>'s warrant still only two line-breaks.  ln_break will be
+ * called with n = 2 for each of multiple successive <br>'s, but ln_break
+ * will only add two line-breaks for the entire run of <br>'s. */
+void ln_break(struct html_context *html_context, int n);
 
-#ifdef CONFIG_ECMASCRIPT
-int do_html_script(unsigned char *attr, unsigned char *html, unsigned char *eof, unsigned char **end, struct part *part);
-#endif
+int get_color(struct html_context *html_context, unsigned char *a, unsigned char *c, color_T *rgb);
 
 #endif
