@@ -1,5 +1,4 @@
 /* HTML tables parser */
-/* $Id: table.c,v 1.35.2.2 2005/04/05 21:08:41 jonas Exp $ */
 
 /* Note that this does *not* fit to the HTML parser infrastructure yet, it has
  * some special custom calling conventions and is managed from
@@ -64,17 +63,19 @@ add_table_bad_html_end(struct table *table, unsigned char *end)
 
 
 static void
-get_bordercolor(unsigned char *a, color_t *rgb)
+get_bordercolor(struct html_context *html_context, unsigned char *a, color_T *rgb)
 {
 	unsigned char *at;
 
-	if (!use_document_fg_colors(global_doc_opts))
+	if (!use_document_fg_colors(html_context->options))
 		return;
 
-	at = get_attr_val(a, "bordercolor");
+	at = get_attr_val(a, "bordercolor", html_context->options);
 	/* Try some other MSIE-specific attributes if any. */
-	if (!at) at = get_attr_val(a, "bordercolorlight");
-	if (!at) at = get_attr_val(a, "bordercolordark");
+	if (!at)
+		at = get_attr_val(a, "bordercolorlight", html_context->options);
+	if (!at)
+		at = get_attr_val(a, "bordercolordark", html_context->options);
 	if (!at) return;
 
 	decode_color(at, strlen(at), rgb);
@@ -82,9 +83,9 @@ get_bordercolor(unsigned char *a, color_t *rgb)
 }
 
 static void
-get_align(unsigned char *attr, int *a)
+get_align(struct html_context *html_context, unsigned char *attr, int *a)
 {
-	unsigned char *al = get_attr_val(attr, "align");
+	unsigned char *al = get_attr_val(attr, "align", html_context->options);
 
 	if (!al) return;
 
@@ -97,9 +98,9 @@ get_align(unsigned char *attr, int *a)
 }
 
 static void
-get_valign(unsigned char *attr, int *a)
+get_valign(struct html_context *html_context, unsigned char *attr, int *a)
 {
-	unsigned char *al = get_attr_val(attr, "valign");
+	unsigned char *al = get_attr_val(attr, "valign", html_context->options);
 
 	if (!al) return;
 
@@ -111,9 +112,10 @@ get_valign(unsigned char *attr, int *a)
 }
 
 static void
-get_column_width(unsigned char *attr, int *width, int sh)
+get_column_width(unsigned char *attr, int *width, int sh,
+                 struct html_context *html_context)
 {
-	unsigned char *al = get_attr_val(attr, "width");
+	unsigned char *al = get_attr_val(attr, "width", html_context->options);
 	int len;
 
 	if (!al) return;
@@ -129,7 +131,7 @@ get_column_width(unsigned char *attr, int *width, int sh)
 		if (!errno && n >= 0 && !*en)
 			*width = WIDTH_RELATIVE - n;
 	} else {
-		int w = get_width(attr, "width", sh);
+		int w = get_width(attr, "width", sh, html_context);
 
 		if (w >= 0) *width = w;
 	}
@@ -137,7 +139,8 @@ get_column_width(unsigned char *attr, int *width, int sh)
 }
 
 static void
-set_table_frame(struct table *table, unsigned char *attr)
+set_table_frame(struct html_context *html_context, struct table *table,
+                unsigned char *attr)
 {
 	unsigned char *al;
 
@@ -148,7 +151,7 @@ set_table_frame(struct table *table, unsigned char *attr)
 
 	table->frame = TABLE_FRAME_BOX;
 
-	al = get_attr_val(attr, "frame");
+	al = get_attr_val(attr, "frame", html_context->options);
 	if (!al) return;
 
 	if (!strcasecmp(al, "void")) table->frame = TABLE_FRAME_VOID;
@@ -166,13 +169,14 @@ set_table_frame(struct table *table, unsigned char *attr)
 }
 
 static void
-set_table_rules(struct table *table, unsigned char *attr)
+set_table_rules(struct html_context *html_context, struct table *table,
+                unsigned char *attr)
 {
 	unsigned char *al;
 
 	table->rules = table->border ? TABLE_RULE_ALL : TABLE_RULE_NONE;
 
-	al = get_attr_val(attr, "rules");
+	al = get_attr_val(attr, "rules", html_context->options);
 	if (!al) return;
 
 	if (!strcasecmp(al, "none")) table->rules = TABLE_RULE_NONE;
@@ -184,13 +188,14 @@ set_table_rules(struct table *table, unsigned char *attr)
 }
 
 static void
-parse_table_attributes(struct table *table, unsigned char *attr, int real)
+parse_table_attributes(struct table *table, unsigned char *attr, int real,
+                       struct html_context *html_context)
 {
-	table->fragment_id = get_attr_val(attr, "id");
+	table->fragment_id = get_attr_val(attr, "id", html_context->options);
 
-	get_bordercolor(attr, &table->bordercolor);
+	get_bordercolor(html_context, attr, &table->bordercolor);
 
-	table->width = get_width(attr, "width", real);
+	table->width = get_width(attr, "width", real, html_context);
 	if (table->width == -1) {
 		table->width = get_html_max_width();
 		table->full_width = 1;
@@ -207,23 +212,26 @@ parse_table_attributes(struct table *table, unsigned char *attr, int real)
 	 * interpreted as the value of the frame attribute. It implies
 	 * rules="all" and some default (non-zero) value for the border
 	 * attribute. */
-	table->border = get_num(attr, "border");
+	table->border = get_num(attr, "border", html_context->options);
 	if (table->border == -1) {
-		table->border = has_attr(attr, "border")
-				|| has_attr(attr, "rules")
-				|| has_attr(attr, "frame");
+		table->border =
+		              has_attr(attr, "border", html_context->options)
+			      || has_attr(attr, "rules", html_context->options)
+			      || has_attr(attr, "frame", html_context->options);
 	}
 
 	if (table->border) {
 		int_upper_bound(&table->border, 2);
 
-		table->cellspacing = get_num(attr, "cellspacing");
+		table->cellspacing = get_num(attr, "cellspacing", html_context->options);
 		int_bounds(&table->cellspacing, 1, 2);
 	}
 
-	set_table_frame(table, attr);
+	set_table_frame(html_context, table, attr);
 
-	table->cellpadding = get_num(attr, "cellpadding");
+	/* TODO: cellpadding may be expressed as a percentage, this is not
+	 * handled yet. */
+	table->cellpadding = get_num(attr, "cellpadding", html_context->options);
 	if (table->cellpadding == -1) {
 		table->vcellpadding = 0;
 		table->cellpadding = !!table->border;
@@ -232,13 +240,13 @@ parse_table_attributes(struct table *table, unsigned char *attr, int real)
 		table->cellpadding = (table->cellpadding >= HTML_CHAR_WIDTH / 2 + 1);
 	}
 
-	set_table_rules(table, attr);
+	set_table_rules(html_context, table, attr);
 
 	table->align = par_format.align;
-	get_align(attr, &table->align);
+	get_align(html_context, attr, &table->align);
 
 	table->bgcolor = par_format.bgcolor;
-	get_bgcolor(attr, &table->bgcolor);
+	get_bgcolor(html_context, attr, &table->bgcolor);
 }
 
 
@@ -500,7 +508,7 @@ skip_table(unsigned char *html, unsigned char *eof)
 
 	while (1) {
 		unsigned char *name;
-		int namelen;
+		int namelen, closing_tag = 0;
 
 		while (html < eof
 		       && (*html != '<'
@@ -510,25 +518,36 @@ skip_table(unsigned char *html, unsigned char *eof)
 
 		if (html >= eof) return eof;
 
+		if (!namelen) continue;
+
+		if (*name == '/') {
+			closing_tag = 1;
+			name++; namelen--;
+			if (!namelen) continue;
+		}
+
+
 		if (!strlcasecmp(name, namelen, "TABLE", 5)) {
-			level++;
-		} else if (!strlcasecmp(name, namelen, "/TABLE", 6)) {
-			level--;
-			if (!level) return html;
+			if (!closing_tag) {
+				level++;
+			} else {
+				level--;
+				if (!level) return html;
+			}
 		}
 	}
 }
 
 struct table *
 parse_table(unsigned char *html, unsigned char *eof, unsigned char **end,
-	    unsigned char *attr, int sh)
+	    unsigned char *attr, int sh, struct html_context *html_context)
 {
 	struct table *table;
 	struct table_cell *cell;
-	unsigned char *t_name, *t_attr, *en;
+	unsigned char *t_attr, *en, *name;
 	unsigned char *l_fragment_id = NULL;
-	color_t last_bgcolor;
-	int t_namelen;
+	color_T last_bgcolor;
+	int namelen;
 	int in_cell = 0;
 	int l_al = ALIGN_LEFT;
 	int l_val = VALIGN_MIDDLE;
@@ -539,13 +558,15 @@ parse_table(unsigned char *html, unsigned char *eof, unsigned char **end,
 	int cols, rows;
 	int col = 0, row = -1;
 	int maxj;
+	int closing_tag, is_header;
+	unsigned char c;
 
 	*end = html;
 
 	table = new_table();
 	if (!table) return NULL;
 
-	parse_table_attributes(table, attr, sh);
+	parse_table_attributes(table, attr, sh, html_context);
 	last_bgcolor = table->bgcolor;
 
 se:
@@ -570,39 +591,53 @@ see:
 		goto se;
 	}
 
-	if (parse_element(html, eof, &t_name, &t_namelen, &t_attr, &en)) {
+	if (parse_element(html, eof, &name, &namelen, &t_attr, &en)) {
 		html++;
 		goto se;
 	}
 
-	if (!strlcasecmp(t_name, t_namelen, "TABLE", 5)) {
-		en = skip_table(en, eof);
+	if (!namelen) goto see;
+
+	if (name[0] == '/') {
+		namelen--;
+		if (!namelen) goto see;
+	       	name++;
+		closing_tag = 1;
+
+	} else {
+		closing_tag = 0;
+	}
+
+	if (!strlcasecmp(name, namelen, "TABLE", 5)) {
+		if (!closing_tag) {
+			en = skip_table(en, eof);
+			goto see;
+
+		} else {
+			if (c_span) new_columns(table, c_span, c_width, c_al, c_val, 1);
+			if (in_cell) CELL(table, col, row)->end = html;
+
+			add_table_bad_html_end(table, html);
+
+			goto scan_done;
+		}
+	}
+
+	if (!strlcasecmp(name, namelen, "CAPTION", 7)) {
+		if (!closing_tag) {
+			add_table_bad_html_end(table, html);
+			if (!table->caption.start)
+				table->caption.start = html;
+
+		} else {
+			if (table->caption.start && !table->caption.end)
+				table->caption.end = html;
+		}
+
 		goto see;
 	}
 
-	if (!strlcasecmp(t_name, t_namelen, "/TABLE", 6)) {
-		if (c_span) new_columns(table, c_span, c_width, c_al, c_val, 1);
-		if (in_cell) CELL(table, col, row)->end = html;
-
-		add_table_bad_html_end(table, html);
-
-		goto scan_done;
-	}
-
-	if (!strlcasecmp(t_name, t_namelen, "CAPTION", 7)) {
-		add_table_bad_html_end(table, html);
-		if (!table->caption.start)
-			table->caption.start = html;
-		goto see;
-	}
-
-	if (!strlcasecmp(t_name, t_namelen, "/CAPTION", 8)) {
-		if (table->caption.start && !table->caption.end)
-			table->caption.end = html;
-		goto see;
-	}
-
-	if (!strlcasecmp(t_name, t_namelen, "COLGROUP", 8)) {
+	if (!strlcasecmp(name, namelen, "COLGROUP", 8)) {
 		if (c_span) new_columns(table, c_span, c_width, c_al, c_val, 1);
 
 		add_table_bad_html_end(table, html);
@@ -610,53 +645,54 @@ see:
 		c_al = ALIGN_TR;
 		c_val = VALIGN_TR;
 		c_width = WIDTH_AUTO;
-		get_align(t_attr, &c_al);
-		get_valign(t_attr, &c_val);
-		get_column_width(t_attr, &c_width, sh);
-		c_span = get_num(t_attr, "span");
-		if (c_span == -1) c_span = 1;
-		else if (c_span > HTML_MAX_COLSPAN) c_span = HTML_MAX_COLSPAN;
+
+		if (!closing_tag) {
+			get_align(html_context, t_attr, &c_al);
+			get_valign(html_context, t_attr, &c_val);
+			get_column_width(t_attr, &c_width, sh, html_context);
+			c_span = get_num(t_attr, "span", html_context->options);
+			if (c_span == -1)
+				c_span = 1;
+			else if (c_span > HTML_MAX_COLSPAN)
+				c_span = HTML_MAX_COLSPAN;
+
+		} else {
+			c_span = 0;
+		}
+
 		goto see;
 	}
 
-	if (!strlcasecmp(t_name, t_namelen, "/COLGROUP", 9)) {
-		if (c_span) new_columns(table, c_span, c_width, c_al, c_val, 1);
-
-		add_table_bad_html_end(table, html);
-
-		c_span = 0;
-		c_al = ALIGN_TR;
-		c_val = VALIGN_TR;
-		c_width = WIDTH_AUTO;
-		goto see;
-	}
-
-	if (!strlcasecmp(t_name, t_namelen, "COL", 3)) {
+	if (!closing_tag && !strlcasecmp(name, namelen, "COL", 3)) {
 		int sp, width, al, val;
 
 		add_table_bad_html_end(table, html);
 
-		sp = get_num(t_attr, "span");
+		sp = get_num(t_attr, "span", html_context->options);
 		if (sp == -1) sp = 1;
 		else if (sp > HTML_MAX_COLSPAN) sp = HTML_MAX_COLSPAN;
 
 		width = c_width;
 		al = c_al;
 		val = c_val;
-		get_align(t_attr, &al);
-		get_valign(t_attr, &val);
-		get_column_width(t_attr, &width, sh);
+		get_align(html_context, t_attr, &al);
+		get_valign(html_context, t_attr, &val);
+		get_column_width(t_attr, &width, sh, html_context);
 		new_columns(table, sp, width, al, val, !!c_span);
 		c_span = 0;
 		goto see;
 	}
 
-	/* /TR /TD /TH */
-	if (t_namelen == 3
-	    && t_name[0] == '/'
-	    && toupper(t_name[1]) == 'T') {
-		unsigned char c = toupper(t_name[2]);
+	/* All following tags have T as first letter. */
+	if (toupper(name[0]) != 'T') goto see;
 
+	name++; namelen--;
+	if (namelen == 0) goto see;
+
+	c = toupper(name[0]);
+
+	/* /TR /TD /TH */
+	if (closing_tag && namelen == 1) {
 		if (c == 'R' || c == 'D' || c == 'H') {
 			if (c_span)
 				new_columns(table, c_span, c_width, c_al, c_val, 1);
@@ -667,14 +703,31 @@ see:
 			}
 
 			add_table_bad_html_end(table, html);
+			goto see;
 		}
 	}
 
-	/* All following tags have T as first letter. */
-	if (toupper(t_name[0]) != 'T') goto see;
+	/* Beyond that point, opening tags only. */
+	if (closing_tag) goto see;
+
+	/* THEAD TBODY TFOOT */
+	if (namelen == 4
+	    && ((!strlcasecmp(name, namelen, "HEAD", 4)) ||
+		(!strlcasecmp(name, namelen, "BODY", 4)) ||
+		(!strlcasecmp(name, namelen, "FOOT", 4)))) {
+		if (c_span) new_columns(table, c_span, c_width, c_al, c_val, 1);
+
+		add_table_bad_html_end(table, html);
+
+		group = 2;
+		goto see;
+	}
+
+	/* Beyond this point, only two letters tags. */
+	if (namelen != 1) goto see;
 
 	/* TR */
-	if (t_namelen == 2 && toupper(t_name[1]) == 'R') {
+	if (c == 'R') {
 		if (c_span) new_columns(table, c_span, c_width, c_al, c_val, 1);
 
 		if (in_cell) {
@@ -688,31 +741,20 @@ see:
 		l_al = ALIGN_LEFT;
 		l_val = VALIGN_MIDDLE;
 		last_bgcolor = table->bgcolor;
-		get_align(t_attr, &l_al);
-		get_valign(t_attr, &l_val);
-		get_bgcolor(t_attr, &last_bgcolor);
-		mem_free_set(&l_fragment_id, get_attr_val(t_attr, "id"));
+		get_align(html_context, t_attr, &l_al);
+		get_valign(html_context, t_attr, &l_val);
+		get_bgcolor(html_context, t_attr, &last_bgcolor);
+		mem_free_set(&l_fragment_id,
+		             get_attr_val(t_attr, "id", html_context->options));
 		row++;
 		col = 0;
 		goto see;
 	}
 
-	/* THEAD TBODY TFOOT */
-	if (t_namelen == 5
-	    && ((!strncasecmp(&t_name[1], "HEAD", 4)) ||
-		(!strncasecmp(&t_name[1], "BODY", 4)) ||
-		(!strncasecmp(&t_name[1], "FOOT", 4)))) {
-		if (c_span) new_columns(table, c_span, c_width, c_al, c_val, 1);
-
-		add_table_bad_html_end(table, html);
-
-		group = 2;
-	}
-
 	/* TD TH */
-	if (t_namelen != 2
-	    || (toupper(t_name[1]) != 'D'
-		&& toupper(t_name[1]) != 'H'))
+	is_header = (c == 'H');
+
+	if (!is_header && c != 'D')
 		goto see;
 
 	if (c_span) new_columns(table, c_span, c_width, c_al, c_val, 1);
@@ -746,13 +788,13 @@ see:
 
 	cell->align = l_al;
 	cell->valign = l_val;
-	cell->fragment_id = get_attr_val(t_attr, "id");
+	cell->fragment_id = get_attr_val(t_attr, "id", html_context->options);
 	if (!cell->fragment_id && l_fragment_id) {
 		cell->fragment_id = l_fragment_id;
 		l_fragment_id = NULL;
 	}
 
-	cell->is_header = (toupper(t_name[1]) == 'H');
+	cell->is_header = is_header;
 	if (cell->is_header) cell->align = ALIGN_CENTER;
 
 	if (group == 1) cell->is_group = 1;
@@ -766,16 +808,16 @@ see:
 
 	cell->bgcolor = last_bgcolor;
 
-	get_align(t_attr, &cell->align);
-	get_valign(t_attr, &cell->valign);
-	get_bgcolor(t_attr, &cell->bgcolor);
+	get_align(html_context, t_attr, &cell->align);
+	get_valign(html_context, t_attr, &cell->valign);
+	get_bgcolor(html_context, t_attr, &cell->bgcolor);
 
-	colspan = get_num(t_attr, "colspan");
+	colspan = get_num(t_attr, "colspan", html_context->options);
 	if (colspan == -1) colspan = 1;
 	else if (!colspan) colspan = -1;
 	else if (colspan > HTML_MAX_COLSPAN) colspan = HTML_MAX_COLSPAN;
 
-	rowspan = get_num(t_attr, "rowspan");
+	rowspan = get_num(t_attr, "rowspan", html_context->options);
 	if (rowspan == -1) rowspan = 1;
 	else if (!rowspan) rowspan = -1;
 	else if (rowspan > HTML_MAX_ROWSPAN) rowspan = HTML_MAX_ROWSPAN;
@@ -786,7 +828,7 @@ see:
 	if (colspan == 1) {
 		int width = WIDTH_AUTO;
 
-		get_column_width(t_attr, &width, sh);
+		get_column_width(t_attr, &width, sh, html_context);
 		if (width != WIDTH_AUTO)
 			set_td_width(table, col, width, 0);
 	}
