@@ -172,7 +172,8 @@ realloc_line(struct html_context *html_context, struct document *document,
 	end->data = ' ';
 	end->attr = 0;
 	set_screen_char_color(end, par_format.bgcolor, 0x0,
-			      0, document->options.color_mode);
+			      COLOR_ENSURE_CONTRAST, /* for bug 461 */
+			      document->options.color_mode);
 
 	for (pos = &line->chars[line->length]; pos < end; pos++) {
 		copy_screen_chars(pos, end, 1);
@@ -250,7 +251,8 @@ clear_hchars(struct html_context *html_context, int x, int y, int width)
 	end->data = ' ';
 	end->attr = 0;
 	set_screen_char_color(end, par_format.bgcolor, 0x0,
-			      0, part->document->options.color_mode);
+			      COLOR_ENSURE_CONTRAST, /* for bug 461 */
+			      part->document->options.color_mode);
 
 	while (pos < end)
 		copy_screen_chars(pos++, end, 1);
@@ -438,7 +440,11 @@ set_hline(struct html_context *html_context, unsigned char *chars, int charslen,
 		}
 	} else {
 		for (; charslen > 0; charslen--, x++, chars++) {
-			part->spaces[x] = (*chars == ' ');
+			if (*chars == NBSP_CHAR) {
+				part->spaces[x] = html_context->options->wrap_nbsp;
+			} else {
+				part->spaces[x] = (*chars == ' ');
+			}
 		}
 	}
 }
@@ -643,7 +649,13 @@ del_chars(struct html_context *html_context, int x, int y)
 	move_links(html_context, x, y, -1, -1);
 }
 
-#define overlap(x) int_max((x).width - (x).rightmargin, 0)
+#if TABLE_LINE_PADDING < 0
+# define overlap_width(x) (x).width
+#else
+# define overlap_width(x) int_min((x).width, \
+	html_context->options->box.width - TABLE_LINE_PADDING)
+#endif
+#define overlap(x) int_max(overlap_width(x) - (x).rightmargin, 0)
 
 static int inline
 split_line_at(struct html_context *html_context, int width)
@@ -1379,7 +1391,7 @@ put_chars(struct html_context *html_context, unsigned char *chars, int charslen)
 	part->cx += charslen;
 	renderer_context.nobreak = 0;
 
-	if (!html_is_preformatted()) {
+	if (!(html_context->options->wrap || html_is_preformatted())) {
 		while (part->cx > overlap(par_format)
 		       && part->cx > par_format.leftmargin) {
 			int x = split_line(html_context);
@@ -1585,8 +1597,8 @@ check_html_form_hierarchy(struct part *part)
 	foreachsafe (fc, next, form_controls) {
 
 		foreach (form, document->forms) {
-			if (fc->position < form->form_num
-			    || form->form_end < fc->position)
+			if (form->form_num <= fc->position
+			    && fc->position <= form->form_end)
 				continue;
 
 			fc->form = form;
@@ -1724,7 +1736,11 @@ html_special(struct html_context *html_context, enum html_special_type c, ...)
 			unsigned long seconds = va_arg(l, unsigned long);
 			unsigned char *t = va_arg(l, unsigned char *);
 
-			document->refresh = init_document_refresh(t, seconds);
+			if (document) {
+				if (document->refresh)
+					done_document_refresh(document->refresh);
+				document->refresh = init_document_refresh(t, seconds);
+			}
 			break;
 		}
 		case SP_COLOR_LINK_LINES:
