@@ -60,7 +60,23 @@ static JSBool input_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval
 /* Each @input_class object must have a @form_class parent.  */
 static const JSClass input_class = {
 	"input", /* here, we unleash ourselves */
-	JSCLASS_HAS_PRIVATE,	/* struct form_state * */
+	/* In instances of @input_class, the private data is not
+	 * actually a pointer, although SMJS assumes so.  Rather, it
+	 * is an integer used as an index to view_state.form_info[].
+	 * This allows ELinks to reallocate form_info[] without
+	 * keeping track of SMJS objects that refer to its elements.
+	 *
+	 * JS_SetPrivate converts private pointers to jsval, and
+	 * JS_GetPrivate converts back.  These conversions assume that
+	 * private pointers are aligned.  Therefore, ELinks must not
+	 * cast the integer directly to void *.  Instead, ELinks takes
+	 * advantage of the fact that the jsval format for private
+	 * pointers is the same as for integers (presumably to make GC
+	 * ignore the pointers).  So when ELinks is initializing the
+	 * private data, it converts the integer to a jsval and from
+	 * there to a pointer, which SMJS then converts back to the
+	 * same jsval.  */
+	JSCLASS_HAS_PRIVATE,
 	JS_PropertyStub, JS_PropertyStub,
 	input_get_property, input_set_property,
 	JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub
@@ -113,7 +129,7 @@ static JSBool input_click(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv
 static JSBool input_focus(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 static JSBool input_select(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 
-static const JSFunctionSpec input_funcs[] = {
+static const spidermonkeyFunctionSpec input_funcs[] = {
 	{ "blur",	input_blur,	0 },
 	{ "click",	input_click,	0 },
 	{ "focus",	input_focus,	0 },
@@ -121,6 +137,18 @@ static const JSFunctionSpec input_funcs[] = {
 	{ NULL }
 };
 
+
+static struct form_state *
+input_get_form_state(JSContext *ctx, JSObject *obj, struct view_state *vs)
+{
+	void *private = JS_GetInstancePrivate(ctx, obj,
+					      (JSClass *) &input_class,
+					      NULL);
+	int n = JSVAL_TO_INT(PRIVATE_TO_JSVAL(private));
+
+	return &vs->form_info[n];
+}
+
 /* @input_class.getProperty */
 static JSBool
 input_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
@@ -151,10 +179,11 @@ input_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 	document = doc_view->document;
-	fs = JS_GetPrivate(ctx, obj); /* from @input_class */
+	fs = input_get_form_state(ctx, obj, vs);
 	fc = find_form_control(document, fs);
 
 	assert(fc);
@@ -248,8 +277,8 @@ input_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 		break;
 
 	default:
-		/* Unrecognized property ID; someone is using the
-		 * object as an array.  SMJS builtin classes (e.g.
+		/* Unrecognized integer property ID; someone is using
+		 * the object as an array.  SMJS builtin classes (e.g.
 		 * js_RegExpClass) just return JS_TRUE in this case
 		 * and leave *@vp unchanged.  Do the same here.
 		 * (Actually not quite the same, as we already used
@@ -290,10 +319,11 @@ input_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 	document = doc_view->document;
-	fs = JS_GetPrivate(ctx, obj); /* from @input_class */
+	fs = input_get_form_state(ctx, obj, vs);
 	fc = find_form_control(document, fs);
 
 	assert(fc);
@@ -351,8 +381,8 @@ input_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 		break;
 
 	default:
-		/* Unrecognized property ID; someone is using the
-		 * object as an array.  SMJS builtin classes (e.g.
+		/* Unrecognized integer property ID; someone is using
+		 * the object as an array.  SMJS builtin classes (e.g.
 		 * js_RegExpClass) just return JS_TRUE in this case.
 		 * Do the same here.  */
 		return JS_TRUE;
@@ -396,11 +426,12 @@ input_click(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 	document = doc_view->document;
 	ses = doc_view->session;
-	fs = JS_GetPrivate(ctx, obj); /* from @input_class */
+	fs = input_get_form_state(ctx, obj, vs);
 
 	assert(fs);
 	fc = find_form_control(document, fs);
@@ -448,11 +479,12 @@ input_focus(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 	document = doc_view->document;
 	ses = doc_view->session;
-	fs = JS_GetPrivate(ctx, obj); /* from @input_class */
+	fs = input_get_form_state(ctx, obj, vs);
 
 	assert(fs);
 	fc = find_form_control(document, fs);
@@ -479,25 +511,27 @@ input_select(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval
 }
 
 static JSObject *
-get_input_object(JSContext *ctx, JSObject *jsform, struct form_state *fs)
+get_input_object(JSContext *ctx, JSObject *jsform, long number)
 {
-	if (!fs->ecmascript_obj) {
-		/* jsform ('form') is input's parent */
-		/* FIXME: That is NOT correct since the real containing element
-		 * should be its parent, but gimme DOM first. --pasky */
-		JSObject *jsinput = JS_NewObject(ctx, (JSClass *) &input_class, NULL, jsform);
+#if 0
+	if (fs->ecmascript_obj)
+		return fs->ecmascript_obj;
+#endif
+	/* jsform ('form') is input's parent */
+	/* FIXME: That is NOT correct since the real containing element
+	 * should be its parent, but gimme DOM first. --pasky */
+	JSObject *jsinput = JS_NewObject(ctx, (JSClass *) &input_class, NULL, jsform);
+	void *private = JSVAL_TO_PRIVATE(INT_TO_JSVAL(number));
 
-		JS_DefineProperties(ctx, jsinput, (JSPropertySpec *) input_props);
-		JS_DefineFunctions(ctx, jsinput, (JSFunctionSpec *) input_funcs);
-		JS_SetPrivate(ctx, jsinput, fs); /* to @input_class */
-		fs->ecmascript_obj = jsinput;
-	}
-	return fs->ecmascript_obj;
+	JS_DefineProperties(ctx, jsinput, (JSPropertySpec *) input_props);
+	spidermonkey_DefineFunctions(ctx, jsinput, input_funcs);
+	JS_SetPrivate(ctx, jsinput, private); /* to @input_class */
+	return jsinput;;
 }
 
 
 static JSObject *
-get_form_control_object(JSContext *ctx, JSObject *jsform, enum form_type type, struct form_state *fs)
+get_form_control_object(JSContext *ctx, JSObject *jsform, enum form_type type, int number)
 {
 	switch (type) {
 		case FC_TEXT:
@@ -510,7 +544,7 @@ get_form_control_object(JSContext *ctx, JSObject *jsform, enum form_type type, s
 		case FC_RESET:
 		case FC_BUTTON:
 		case FC_HIDDEN:
-			return get_input_object(ctx, jsform, fs);
+			return get_input_object(ctx, jsform, (long)number);
 
 		case FC_TEXTAREA:
 		case FC_SELECT:
@@ -539,7 +573,7 @@ static const JSClass form_elements_class = {
 static JSBool form_elements_item(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 static JSBool form_elements_namedItem(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 
-static const JSFunctionSpec form_elements_funcs[] = {
+static const spidermonkeyFunctionSpec form_elements_funcs[] = {
 	{ "item",		form_elements_item,		1 },
 	{ "namedItem",		form_elements_namedItem,	1 },
 	{ NULL }
@@ -581,10 +615,12 @@ form_elements_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 	document = doc_view->document;
-	form_view = JS_GetPrivate(ctx, parent_form); /* from @form_class */
+	form_view = JS_GetInstancePrivate(ctx, parent_form,
+					  (JSClass *) &form_class, NULL);
 	form = find_form_by_form_view(document, form_view);
 
 	if (JSVAL_IS_STRING(id)) {
@@ -637,10 +673,12 @@ form_elements_item(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 	document = doc_view->document;
-	form_view = JS_GetPrivate(ctx, parent_form); /* from @form_class */
+	form_view = JS_GetInstancePrivate(ctx, parent_form,
+					  (JSClass *) &form_class, NULL);
 	form = find_form_by_form_view(document, form_view);
 
 	if (argc != 1)
@@ -653,10 +691,13 @@ form_elements_item(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval
 	foreach (fc, form->items) {
 		counter++;
 		if (counter == index) {
-			JSObject *fcobj = get_form_control_object(ctx, parent_form, fc->type, find_form_state(doc_view, fc));
+			struct form_state *fs = find_form_state(doc_view, fc);
 
-			if (fcobj) {
-				object_to_jsval(ctx, rval, fcobj);
+			if (fs) {
+				JSObject *fcobj = get_form_control_object(ctx, parent_form, fc->type, fc->g_ctrl_num);
+
+				if (fcobj)
+					object_to_jsval(ctx, rval, fcobj);
 			}
 			break;
 		}
@@ -691,10 +732,12 @@ form_elements_namedItem(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv,
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 	document = doc_view->document;
-	form_view = JS_GetPrivate(ctx, parent_form); /* from @form_class */
+	form_view = JS_GetInstancePrivate(ctx, parent_form,
+					  (JSClass *) &form_class, NULL);
 	form = find_form_by_form_view(document, form_view);
 
 	if (argc != 1)
@@ -708,10 +751,13 @@ form_elements_namedItem(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv,
 
 	foreach (fc, form->items) {
 		if (fc->name && !strcasecmp(string, fc->name)) {
-			JSObject *fcobj = get_form_control_object(ctx, parent_form, fc->type, find_form_state(doc_view, fc));
+			struct form_state *fs = find_form_state(doc_view, fc);
 
-			if (fcobj) {
-				object_to_jsval(ctx, rval, fcobj);
+			if (fs) {
+				JSObject *fcobj = get_form_control_object(ctx, parent_form, fc->type, fc->g_ctrl_num);
+
+				if (fcobj)
+					object_to_jsval(ctx, rval, fcobj);
 			}
 			break;
 		}
@@ -758,7 +804,7 @@ static const JSPropertySpec form_props[] = {
 static JSBool form_reset(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 static JSBool form_submit(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 
-static const JSFunctionSpec form_funcs[] = {
+static const spidermonkeyFunctionSpec form_funcs[] = {
 	{ "reset",	form_reset,	0 },
 	{ "submit",	form_submit,	0 },
 	{ NULL }
@@ -788,9 +834,10 @@ form_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
-	fv = JS_GetPrivate(ctx, obj); /* from @form_class */
+	fv = JS_GetInstancePrivate(ctx, obj, (JSClass *) &form_class, NULL);
 	form = find_form_by_form_view(doc_view->document, fv);
 
 	assert(form);
@@ -802,15 +849,17 @@ form_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 		string = jsval_to_string(ctx, &id);
 		foreach (fc, form->items) {
 			JSObject *fcobj = NULL;
+			struct form_state *fs;
 
 			if (!fc->name || strcasecmp(string, fc->name))
 				continue;
 
-			fcobj = get_form_control_object(ctx, obj, fc->type, find_form_state(doc_view, fc));
-			if (fcobj) {
-				object_to_jsval(ctx, vp, fcobj);
-			} else {
-				undef_to_jsval(ctx, vp);
+			undef_to_jsval(ctx, vp);
+			fs = find_form_state(doc_view, fc);
+			if (fs) {
+				fcobj = get_form_control_object(ctx, obj, fc->type, fc->g_ctrl_num);
+				if (fcobj)
+					object_to_jsval(ctx, vp, fcobj);
 			}
 			break;
 		}
@@ -833,7 +882,8 @@ form_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 		JSObject *jsform_elems = JS_NewObject(ctx, (JSClass *) &form_elements_class, NULL, obj);
 
 		JS_DefineProperties(ctx, jsform_elems, (JSPropertySpec *) form_elements_props);
-		JS_DefineFunctions(ctx, jsform_elems, (JSFunctionSpec *) form_elements_funcs);
+		spidermonkey_DefineFunctions(ctx, jsform_elems,
+					     form_elements_funcs);
 		object_to_jsval(ctx, vp, jsform_elems);
 		/* SM will cache this property value for us so we create this
 		 * just once per form. */
@@ -882,8 +932,8 @@ form_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 		break;
 
 	default:
-		/* Unrecognized property ID; someone is using the
-		 * object as an array.  SMJS builtin classes (e.g.
+		/* Unrecognized integer property ID; someone is using
+		 * the object as an array.  SMJS builtin classes (e.g.
 		 * js_RegExpClass) just return JS_TRUE in this case
 		 * and leave *@vp unchanged.  Do the same here.
 		 * (Actually not quite the same, as we already used
@@ -918,9 +968,10 @@ form_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
-	fv = JS_GetPrivate(ctx, obj); /* from @form_class */
+	fv = JS_GetInstancePrivate(ctx, obj, (JSClass *) &form_class, NULL);
 	form = find_form_by_form_view(doc_view->document, fv);
 
 	assert(form);
@@ -963,8 +1014,8 @@ form_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 		break;
 
 	default:
-		/* Unrecognized property ID; someone is using the
-		 * object as an array.  SMJS builtin classes (e.g.
+		/* Unrecognized integer property ID; someone is using
+		 * the object as an array.  SMJS builtin classes (e.g.
 		 * js_RegExpClass) just return JS_TRUE in this case.
 		 * Do the same here.  */
 		break;
@@ -992,9 +1043,10 @@ form_reset(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
-	fv = JS_GetPrivate(ctx, obj); /* from @form_class */
+	fv = JS_GetInstancePrivate(ctx, obj, (JSClass *) &form_class, argv);
 	form = find_form_by_form_view(doc_view->document, fv);
 
 	assert(form);
@@ -1027,10 +1079,11 @@ form_submit(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 	ses = doc_view->session;
-	fv = JS_GetPrivate(ctx, obj); /* from @form_class */
+	fv = JS_GetInstancePrivate(ctx, obj, (JSClass *) &form_class, argv);
 	form = find_form_by_form_view(doc_view->document, fv);
 
 	assert(form);
@@ -1044,17 +1097,19 @@ form_submit(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 JSObject *
 get_form_object(JSContext *ctx, JSObject *jsdoc, struct form_view *fv)
 {
-	if (!fv->ecmascript_obj) {
-		/* jsdoc ('document') is fv's parent */
-		/* FIXME: That is NOT correct since the real containing element
-		 * should be its parent, but gimme DOM first. --pasky */
-		JSObject *jsform = JS_NewObject(ctx, (JSClass *) &form_class, NULL, jsdoc);
+#if 0
+	if (fv->ecmascript_obj)
+		return fv->ecmascript_obj;
+#endif
+	/* jsdoc ('document') is fv's parent */
+	/* FIXME: That is NOT correct since the real containing element
+	 * should be its parent, but gimme DOM first. --pasky */
+	JSObject *jsform = JS_NewObject(ctx, (JSClass *) &form_class, NULL, jsdoc);
 
-		JS_DefineProperties(ctx, jsform, (JSPropertySpec *) form_props);
-		JS_DefineFunctions(ctx, jsform, (JSFunctionSpec *) form_funcs);
-		JS_SetPrivate(ctx, jsform, fv); /* to @form_class */
-		fv->ecmascript_obj = jsform;
-	}
+	JS_DefineProperties(ctx, jsform, (JSPropertySpec *) form_props);
+	spidermonkey_DefineFunctions(ctx, jsform, form_funcs);
+	JS_SetPrivate(ctx, jsform, fv); /* to @form_class */
+	fv->ecmascript_obj = jsform;
 	return fv->ecmascript_obj;
 }
 static JSBool forms_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp);
@@ -1071,7 +1126,7 @@ const JSClass forms_class = {
 static JSBool forms_item(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 static JSBool forms_namedItem(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 
-const JSFunctionSpec forms_funcs[] = {
+const spidermonkeyFunctionSpec forms_funcs[] = {
 	{ "item",		forms_item,		1 },
 	{ "namedItem",		forms_namedItem,	1 },
 	{ NULL }
@@ -1085,6 +1140,29 @@ const JSPropertySpec forms_props[] = {
 	{ NULL }
 };
 
+/* Find the form whose name is @name, which should normally be a
+ * string (but might not be).  If found, set *rval = the DOM
+ * object.  If not found, leave *rval unchanged.  */
+static void
+find_form_by_name(JSContext *ctx, JSObject *jsdoc,
+		  struct document_view *doc_view,
+		  jsval name, jsval *rval)
+{
+	unsigned char *string = jsval_to_string(ctx, &name);
+	struct form *form;
+
+	if (!*string)
+		return;
+
+	foreach (form, doc_view->document->forms) {
+		if (form->name && !strcasecmp(string, form->name)) {
+			object_to_jsval(ctx, rval, get_form_object(ctx, jsdoc,
+					find_form_view(doc_view, form)));
+			break;
+		}
+	}
+}
+
 /* @forms_class.getProperty */
 static JSBool
 forms_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
@@ -1107,12 +1185,19 @@ forms_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 	document = doc_view->document;
 
 	if (JSVAL_IS_STRING(id)) {
-		forms_namedItem(ctx, obj, 1, &id, vp);
+		/* When SMJS evaluates forms.namedItem("foo"), it first
+		 * calls forms_get_property with id = JSString "namedItem"
+		 * and *vp = JSObject JSFunction forms_namedItem.
+		 * If we don't find a form whose name is id,
+		 * we must leave *vp unchanged here, to avoid
+		 * "TypeError: forms.namedItem is not a function".  */
+		find_form_by_name(ctx, parent_doc, doc_view, id, vp);
 		return JS_TRUE;
 	}
 
@@ -1151,7 +1236,8 @@ forms_item(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 
 	if (argc != 1)
 		return JS_TRUE;
@@ -1179,9 +1265,6 @@ forms_namedItem(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *r
 	JSObject *parent_win;	/* instance of @window_class */
 	struct view_state *vs;
 	struct document_view *doc_view;
-	struct document *document;
-	struct form *form;
-	unsigned char *string;
 
 	if (!JS_InstanceOf(ctx, obj, (JSClass *) &forms_class, argv)) return JS_FALSE;
 	parent_doc = JS_GetParent(ctx, obj);
@@ -1191,26 +1274,14 @@ forms_namedItem(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *r
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
-	document = doc_view->document;
 
 	if (argc != 1)
 		return JS_TRUE;
 
 	undef_to_jsval(ctx, rval);
-
-	string = jsval_to_string(ctx, &argv[0]);
-	if (!*string)
-		return JS_TRUE;
-
-	foreach (form, document->forms) {
-		if (form->name && !strcasecmp(string, form->name)) {
-			object_to_jsval(ctx, rval, get_form_object(ctx, parent_doc,
-					find_form_view(doc_view, form)));
-			break;
-		}
-	}
-
+	find_form_by_name(ctx, parent_doc, doc_view, argv[0], rval);
 	return JS_TRUE;
 }
