@@ -149,7 +149,7 @@ static int
 check_uri_file(unsigned char *name)
 {
 	/* Check POST_CHAR etc ... */
-	static const unsigned char chars[] = POST_CHAR_S "#?";
+	static const unsigned char chars[] = POST_CHAR_S "#";
 
 	return strcspn(name, chars);
 }
@@ -161,6 +161,7 @@ encode_file_uri_string(struct string *string, unsigned char *uristring)
 	int filenamelen = check_whether_file_exists(uristring);
 
 	encode_uri_string(string, uristring, filenamelen, 0);
+	if (filenamelen > 0) add_to_string(string, uristring + filenamelen);
 }
 
 
@@ -241,11 +242,18 @@ parse_uri(struct uri *uri, unsigned char *uristring)
 
 	} else if (uri->protocol == PROTOCOL_FILE) {
 		int datalen = check_uri_file(prefix_end);
+		unsigned char *frag_or_post = prefix_end + datalen;
 
 		/* Extract the fragment part. */
-		if (datalen >= 0 && prefix_end[datalen] == '#') {
-			uri->fragment = prefix_end + datalen + 1;
-			uri->fragmentlen = strlen(uri->fragment);
+		if (datalen >= 0) {
+			if (*frag_or_post == '#') {
+				uri->fragment = frag_or_post + 1;
+				uri->fragmentlen = strcspn(uri->fragment, POST_CHAR_S);
+				frag_or_post = uri->fragment + uri->fragmentlen;
+			}
+			if (*frag_or_post == POST_CHAR) {
+				uri->post = frag_or_post + 1;
+			}
 		} else {
 			datalen = strlen(prefix_end);
 		}
@@ -550,7 +558,7 @@ add_uri_to_string(struct string *string, struct uri *uri,
 	}
 
 	/* Only add slash if we need to separate */
-	if ((wants(URI_DATA) || wants(URI_POST))
+	if ((wants(URI_DATA) || wants(URI_POST) || components == URI_HTTP_REFERRER_HOST)
 	    && wants(~(URI_DATA | URI_PORT))
 	    && get_protocol_need_slash_after_host(uri->protocol))
 		add_char_to_string(string, '/');
