@@ -1,5 +1,4 @@
 /* Ruby scripting hooks */
-/* $Id: hooks.c,v 1.7.2.3 2005/09/13 15:41:24 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -9,11 +8,13 @@
 
 #include "elinks.h"
 
+#include "cache/cache.h"
+#include "main/event.h"
 #include "protocol/uri.h"
-#include "sched/event.h"
-#include "sched/session.h"
 #include "scripting/ruby/core.h"
 #include "scripting/ruby/hooks.h"
+#include "session/location.h"
+#include "session/session.h"
 #include "util/string.h"
 
 
@@ -144,22 +145,22 @@ script_hook_follow_url(va_list ap, void *data)
 static enum evhook_status
 script_hook_pre_format_html(va_list ap, void *data)
 {
-	unsigned char **html = va_arg(ap, unsigned char **);
-	int *html_len = va_arg(ap, int *);
 	struct session *ses = va_arg(ap, struct session *);
-	unsigned char *url = va_arg(ap, unsigned char *);
+	struct cache_entry *cached = va_arg(ap, struct cache_entry *);
+	struct fragment *fragment = get_cache_fragment(cached);
+	unsigned char *url = struri(cached->uri);
 	int error;
 	VALUE args[2];
 	VALUE result;
 
-	evhook_use_params(url && ses);
+	evhook_use_params(ses && cached);
 
-	if (*html == NULL || *html_len == 0)
+	if (!cached->length || !*fragment->data)
 		return EVENT_HOOK_STATUS_NEXT;
 
 	args[0] = rb_str_new2(url);
 	/* FIXME: Use html_len */
-	args[1] = rb_str_new2(*html);
+	args[1] = rb_str_new2(fragment->data);
 
 	result = erb_protected_method_call("pre_format_html_hook", 2, args, &error);
 	if (error) {
@@ -170,13 +171,11 @@ script_hook_pre_format_html(va_list ap, void *data)
 	switch (rb_type(result)) {
 	case T_STRING:
 	{
-		unsigned char *new_html;
+		int len = RSTRING(result)->len;
+
+		add_fragment(cached, 0, RSTRING(result)->ptr, len);
+		normalize_cache_entry(cached, len);
 
-		new_html = memacpy(RSTRING(result)->ptr, RSTRING(result)->len);
-		if (new_html) {
-			*html_len = RSTRING(result)->len;
-			*html = new_html;
-		}
 		break;
 	}
 	case T_NIL:
@@ -248,11 +247,11 @@ script_hook_quit(va_list ap, void *data)
 }
 
 struct event_hook_info ruby_scripting_hooks[] = {
-	{ "goto-url",		script_hook_goto_url,		NULL },
-	{ "follow-url",		script_hook_follow_url,		NULL },
-	{ "pre-format-html",	script_hook_pre_format_html,	NULL },
-	{ "get-proxy",		script_hook_get_proxy,		NULL },
-	{ "quit",		script_hook_quit,		NULL },
+	{ "goto-url",        0, script_hook_goto_url,        NULL },
+	{ "follow-url",      0, script_hook_follow_url,      NULL },
+	{ "pre-format-html", 0, script_hook_pre_format_html, NULL },
+	{ "get-proxy",       0, script_hook_get_proxy,       NULL },
+	{ "quit",            0, script_hook_quit,            NULL },
 
 	NULL_EVENT_HOOK_INFO,
 };
