@@ -45,7 +45,8 @@ cache_entry_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	if (!JS_InstanceOf(ctx, obj, (JSClass *) &cache_entry_class, NULL))
 		return JS_FALSE;
 
-	cached = JS_GetPrivate(ctx, obj); /* from @cache_entry_class */
+	cached = JS_GetInstancePrivate(ctx, obj, 
+				       (JSClass *) &cache_entry_class, NULL);
 
 	if (!cache_entry_is_valid(cached)) return JS_FALSE;
 
@@ -86,8 +87,8 @@ cache_entry_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 
 		return JS_TRUE;
 	default:
-		/* Unrecognized property ID; someone is using the
-		 * object as an array.  SMJS builtin classes (e.g.
+		/* Unrecognized integer property ID; someone is using
+		 * the object as an array.  SMJS builtin classes (e.g.
 		 * js_RegExpClass) just return JS_TRUE in this case
 		 * and leave *@vp unchanged.  Do the same here.
 		 * (Actually not quite the same, as we already used
@@ -108,7 +109,8 @@ cache_entry_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	if (!JS_InstanceOf(ctx, obj, (JSClass *) &cache_entry_class, NULL))
 		return JS_FALSE;
 
-	cached = JS_GetPrivate(ctx, obj); /* from @cache_entry_class */
+	cached = JS_GetInstancePrivate(ctx, obj,
+				       (JSClass *) &cache_entry_class, NULL);
 
 	if (!cache_entry_is_valid(cached)) return JS_FALSE;
 
@@ -143,8 +145,8 @@ cache_entry_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 		return JS_TRUE;
 	}
 	default:
-		/* Unrecognized property ID; someone is using the
-		 * object as an array.  SMJS builtin classes (e.g.
+		/* Unrecognized integer property ID; someone is using
+		 * the object as an array.  SMJS builtin classes (e.g.
 		 * js_RegExpClass) just return JS_TRUE in this case.
 		 * Do the same here.  */
 		return JS_TRUE;
@@ -160,7 +162,8 @@ cache_entry_finalize(JSContext *ctx, JSObject *obj)
 	assert(JS_InstanceOf(ctx, obj, (JSClass *) &cache_entry_class, NULL));
 	if_assert_failed return;
 
-	cached = JS_GetPrivate(ctx, obj); /* from @cache_entry_class */
+	cached = JS_GetInstancePrivate(ctx, obj,
+				       (JSClass *) &cache_entry_class, NULL);
 
 	if (!cached) return;
 
