@@ -41,7 +41,8 @@ view_state_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	if (!JS_InstanceOf(ctx, obj, (JSClass *) &view_state_class, NULL))
 		return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, obj); /* from @view_state_class */
+	vs = JS_GetInstancePrivate(ctx, obj,
+				   (JSClass *) &view_state_class, NULL);
 
 	undef_to_jsval(ctx, vp);
 
@@ -59,8 +60,8 @@ view_state_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 
 		return JS_TRUE;
 	default:
-		/* Unrecognized property ID; someone is using the
-		 * object as an array.  SMJS builtin classes (e.g.
+		/* Unrecognized integer property ID; someone is using
+		 * the object as an array.  SMJS builtin classes (e.g.
 		 * js_RegExpClass) just return JS_TRUE in this case
 		 * and leave *@vp unchanged.  Do the same here.
 		 * (Actually not quite the same, as we already used
@@ -81,7 +82,8 @@ view_state_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	if (!JS_InstanceOf(ctx, obj, (JSClass *) &view_state_class, NULL))
 		return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, obj); /* from @view_state_class */
+	vs = JS_GetInstancePrivate(ctx, obj,
+				   (JSClass *) &view_state_class, NULL);
 
 	if (!JSVAL_IS_INT(id))
 		return JS_FALSE;
@@ -93,8 +95,8 @@ view_state_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 		return JS_TRUE;
 	}
 	default:
-		/* Unrecognized property ID; someone is using the
-		 * object as an array.  SMJS builtin classes (e.g.
+		/* Unrecognized integer property ID; someone is using
+		 * the object as an array.  SMJS builtin classes (e.g.
 		 * js_RegExpClass) just return JS_TRUE in this case.
 		 * Do the same here.  */
 		return JS_TRUE;
